@@ -141,11 +141,11 @@
 
 struct glfs;
 
-struct _upcall_entry_t {
+struct _upcall_entry {
         struct list_head  upcall_list;
         struct gf_upcall  upcall_data;
 };
-typedef struct _upcall_entry_t upcall_entry;
+typedef struct _upcall_entry upcall_entry;
 
 typedef int (*glfs_init_cbk) (struct glfs *fs, int ret);
 
@@ -215,6 +215,7 @@ struct glfs_fd {
 	struct list_head   entries;
 	gf_dirent_t       *next;
 	struct dirent     *readdirbuf;
+        gf_lkowner_t       lk_owner;
 };
 
 /* glfs object handle introduced for the alternate gfapi implementation based
@@ -373,7 +374,6 @@ int glfs_loc_touchup (loc_t *loc)
 void glfs_iatt_to_stat (struct glfs *fs, struct iatt *iatt, struct stat *stat);
 int glfs_loc_link (loc_t *loc, struct iatt *iatt);
 int glfs_loc_unlink (loc_t *loc);
-dict_t *dict_for_key_value (const char *name, const char *value, size_t size);
 int glfs_getxattr_process (void *value, size_t size, dict_t *xattr,
 			   const char *name);
 
@@ -490,4 +490,10 @@ glfd_entry_next (struct glfs_fd *glfd, int plus);
 void
 gf_dirent_to_dirent (gf_dirent_t *gf_dirent, struct dirent *dirent);
 
+/*
+ * Nobody needs this call at all yet except for the test script.
+ */
+int glfs_ipc (glfs_fd_t *fd, int cmd,  void *xd_in, void **xd_out) __THROW
+        GFAPI_PRIVATE(glfs_ipc, 3.12.0);
+
 #endif /* !_GLFS_INTERNAL_H */
