@@ -515,7 +515,7 @@ cli_usage_out (const char *usage)
         if (!usage || usage[0] == '\0')
                 return -1;
 
-        cli_err ("Usage: %s", usage);
+        cli_err ("\nUsage:\n%s\n", usage);
         return 0;
 }
 
@@ -722,6 +722,9 @@ main (int argc, char *argv[])
         int                ret = -1;
         glusterfs_ctx_t   *ctx = NULL;
 
+       mem_pools_init_early ();
+       mem_pools_init_late ();
+
         ctx = glusterfs_ctx_new ();
         if (!ctx)
                 return ENOMEM;
@@ -786,6 +789,8 @@ main (int argc, char *argv[])
 out:
 //        glusterfs_ctx_destroy (ctx);
 
+        mem_pools_fini ();
+
         return ret;
 }
 
