@@ -44,6 +44,7 @@ void trap (void);
 #include "iatt.h"
 #include "uuid.h"
 #include "libglusterfs-messages.h"
+#include "protocol-common.h"
 
 #define STRINGIFY(val) #val
 #define TOSTRING(val) STRINGIFY(val)
@@ -101,7 +102,7 @@ void trap (void);
 #define GF_CLNT_INSECURE_PORT_CEILING (GF_IANA_PRIV_PORTS_START - 1)
 #define GF_PORT_MAX 65535
 #define GF_PORT_ARRAY_SIZE ((GF_PORT_MAX + 7) / 8)
-
+#define GF_LOCK_TIMER 180
 #define GF_MINUTE_IN_SECONDS 60
 #define GF_HOUR_IN_SECONDS (60*60)
 #define GF_DAY_IN_SECONDS (24*60*60)
@@ -113,6 +114,10 @@ void trap (void);
 /* Default value of signing waiting time to sign a file for bitrot */
 #define SIGNING_TIMEOUT "120"
 
+/* xxhash */
+#define GF_XXH64_DIGEST_LENGTH 8
+#define GF_XXHSUM64_DEFAULT_SEED 0
+
 /* Shard */
 #define GF_XATTR_SHARD_FILE_SIZE  "trusted.glusterfs.shard.file-size"
 #define SHARD_ROOT_GFID "be318638-e8a0-4c6d-977d-7a937aa84806"
@@ -126,6 +131,11 @@ void trap (void);
 
 #define GF_PERCENTAGE(val, total) (((val)*100)/(total))
 
+/* pthread related */
+#define GF_THREAD_NAMEMAX 9
+#define GF_THREAD_NAME_PREFIX "gluster"
+#define GF_THREAD_NAME_PREFIX_LEN 7
+
 enum _gf_boolean
 {
 	_gf_false = 0,
@@ -678,12 +688,15 @@ gf_time_fmt (char *dst, size_t sz_dst, time_t utime, unsigned int fmt)
         static gf_timefmts timefmt_last = (gf_timefmts) - 1;
         static const char **fmts;
         static const char **zeros;
-        struct tm tm;
+        struct tm tm, *res;
+        int localtime = 0;
 
         if (timefmt_last == (gf_timefmts) - 1)
                 _gf_timestuff (&timefmt_last, &fmts, &zeros);
         if (timefmt_last < fmt) fmt = gf_timefmt_default;
-        if (utime && gmtime_r (&utime, &tm) != NULL) {
+        localtime = gf_log_get_localtime ();
+        res = localtime ? localtime_r (&utime, &tm) : gmtime_r (&utime, &tm);
+        if (utime && res != NULL) {
                 strftime (dst, sz_dst, fmts[fmt], &tm);
         } else {
                 strncpy (dst, "N/A", sz_dst);
@@ -823,12 +836,16 @@ int gf_get_hostname_from_ip (char *client_ip, char **hostname);
 gf_boolean_t gf_is_local_addr (char *hostname);
 gf_boolean_t gf_is_same_address (char *host1, char *host2);
 void md5_wrapper(const unsigned char *data, size_t len, char *md5);
+void gf_xxh64_wrapper(const unsigned char *data, size_t len,
+                      unsigned long long seed, char *xxh64);
 int gf_set_timestamp  (const char *src, const char* dest);
 
 int gf_thread_create (pthread_t *thread, const pthread_attr_t *attr,
-                      void *(*start_routine)(void *), void *arg);
+                      void *(*start_routine)(void *), void *arg,
+                      const char *name);
 int gf_thread_create_detached (pthread_t *thread,
-                      void *(*start_routine)(void *), void *arg);
+                      void *(*start_routine)(void *), void *arg,
+                      const char *name);
 gf_boolean_t
 gf_is_pid_running (int pid);
 gf_boolean_t
@@ -908,9 +925,22 @@ gf_bits_index (uint64_t n);
 const char*
 gf_fop_string (glusterfs_fop_t fop);
 
+int
+gf_fop_int (char *fop);
+
 char *
 get_ip_from_addrinfo (struct addrinfo *addr, char **ip);
 
 int
 close_fds_except (int *fdv, size_t count);
+
+int
+gf_getgrouplist (const char *user, gid_t group, gid_t **groups);
+
+int
+glusterfs_compute_sha256 (const unsigned char *content, size_t size,
+                          char *sha256_hash);
+char*
+get_struct_variable (int mem_num, gf_gsync_status_t *sts_val);
+
 #endif /* _COMMON_UTILS_H */
