@@ -34,7 +34,7 @@ static int32_t
 worm_open (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
            fd_t *fd, dict_t *xdata)
 {
-        if (is_readonly_or_worm_enabled (this) &&
+        if (is_readonly_or_worm_enabled (frame, this) &&
             (flags & (O_WRONLY | O_RDWR | O_APPEND | O_TRUNC))) {
                 STACK_UNWIND_STRICT (open, frame, -1, EROFS, NULL, NULL);
                 return 0;
@@ -55,7 +55,7 @@ worm_link (call_frame_t *frame, xlator_t *this, loc_t *oldloc, loc_t *newloc,
 
         priv = this->private;
         GF_ASSERT (priv);
-        if (is_readonly_or_worm_enabled (this))
+        if (is_readonly_or_worm_enabled (frame, this))
                 goto out;
         if (!priv->worm_file || (frame->root->pid < 0)) {
                 op_errno = 0;
@@ -91,7 +91,7 @@ worm_unlink (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
 
         priv = this->private;
         GF_ASSERT (priv);
-        if (is_readonly_or_worm_enabled (this)) {
+        if (is_readonly_or_worm_enabled (frame, this)) {
                 goto out;
         }
         if (!priv->worm_file || (frame->root->pid < 0)) {
@@ -127,7 +127,7 @@ worm_rename (call_frame_t *frame, xlator_t *this,
 
         priv = this->private;
         GF_ASSERT (priv);
-        if (is_readonly_or_worm_enabled (this))
+        if (is_readonly_or_worm_enabled (frame, this))
                 goto out;
         if (!priv->worm_file || (frame->root->pid < 0)) {
                 op_errno = 0;
@@ -137,10 +137,24 @@ worm_rename (call_frame_t *frame, xlator_t *this,
         gf_uuid_copy (oldloc->gfid, oldloc->inode->gfid);
         if (is_wormfile (this, _gf_false, oldloc)) {
                 op_errno = 0;
-                goto out;
+                goto check_newloc;
         }
         op_errno = gf_worm_state_transition (this, _gf_false, oldloc,
-                                           GF_FOP_RENAME);
+                                             GF_FOP_RENAME);
+
+        if (op_errno == 0) {
+check_newloc:
+                if (newloc->inode != NULL) {
+                        gf_uuid_copy (newloc->gfid, newloc->inode->gfid);
+                        if (is_wormfile (this, _gf_false, newloc)) {
+                                op_errno = 0;
+                                goto out;
+                        }
+                        op_errno = gf_worm_state_transition (this, _gf_false,
+                                                             newloc,
+                                                             GF_FOP_RENAME);
+                }
+        }
 
 out:
         if (op_errno)
@@ -163,7 +177,7 @@ worm_truncate (call_frame_t *frame, xlator_t *this, loc_t *loc, off_t offset,
 
         priv = this->private;
         GF_ASSERT (priv);
-        if (is_readonly_or_worm_enabled (this))
+        if (is_readonly_or_worm_enabled (frame, this))
                 goto out;
         if (!priv->worm_file || (frame->root->pid < 0)) {
                 op_errno = 0;
@@ -198,7 +212,7 @@ worm_ftruncate (call_frame_t *frame, xlator_t *this, fd_t *fd, off_t offset,
 
         priv = this->private;
         GF_ASSERT (priv);
-        if (is_readonly_or_worm_enabled (this))
+        if (is_readonly_or_worm_enabled (frame, this))
                 goto out;
         if (!priv->worm_file || (frame->root->pid < 0)) {
                 op_errno = 0;
