@@ -441,6 +441,7 @@ glusterd_get_args_from_dict (gf_getspec_req *args, peer_info_t *peerinfo,
         int        client_min_op_version = 1;
         int32_t    ret                   = -1;
         xlator_t  *this                  = NULL;
+        char      *name                  = NULL;
 
         this = THIS;
         GF_ASSERT (this);
@@ -485,14 +486,18 @@ glusterd_get_args_from_dict (gf_getspec_req *args, peer_info_t *peerinfo,
                 goto out;
         }
 
-        ret = dict_get_str (dict, "brick_name",
-                            brick_name);
+        ret = dict_get_str (dict, "brick_name", &name);
         if (ret) {
                 gf_msg_debug (this->name, 0,
                         "No brick name present");
                 ret = 0;
                 goto out;
         }
+        *brick_name = gf_strdup(name);
+        if (*brick_name == NULL) {
+                ret = -1;
+                goto out;
+        }
 
         gf_msg_debug (this->name, 0, "brick_name = %s", *brick_name);
 out:
@@ -653,6 +658,15 @@ glusterd_create_missed_snap (glusterd_missed_snap_info *missed_snapinfo,
         }
 
         brickinfo->snap_status = 0;
+        ret = glusterd_brick_start (snap_vol, brickinfo, _gf_false);
+        if (ret) {
+                gf_msg (this->name, GF_LOG_WARNING, 0,
+                        GD_MSG_BRICK_DISCONNECTED, "starting the "
+                        "brick %s:%s for the snap %s failed",
+                        brickinfo->hostname, brickinfo->path,
+                        snap->snapname);
+                goto out;
+        }
         ret = glusterd_store_volinfo (snap_vol,
                                       GLUSTERD_VOLINFO_VER_AC_NONE);
         if (ret) {
@@ -663,15 +677,6 @@ glusterd_create_missed_snap (glusterd_missed_snap_info *missed_snapinfo,
                 goto out;
         }
 
-        ret = glusterd_brick_start (snap_vol, brickinfo, _gf_false);
-        if (ret) {
-                gf_msg (this->name, GF_LOG_WARNING, 0,
-                        GD_MSG_BRICK_DISCONNECTED, "starting the "
-                        "brick %s:%s for the snap %s failed",
-                        brickinfo->hostname, brickinfo->path,
-                        snap->snapname);
-                goto out;
-        }
 out:
         if (device)
                 GF_FREE (device);
@@ -943,6 +948,8 @@ fail:
         if (spec_fd > 0)
                 sys_close (spec_fd);
 
+        GF_FREE(brick_name);
+
         rsp.op_ret   = ret;
 
         if (op_errno)
