@@ -239,7 +239,8 @@ pmap_assign_port (xlator_t *this, int old_port, const char *path)
 
         if (old_port) {
                 ret = pmap_registry_remove (this, 0, path,
-                                            GF_PMAP_PORT_BRICKSERVER, NULL);
+                                            GF_PMAP_PORT_BRICKSERVER, NULL,
+                                            _gf_false);
                 if (ret) {
                         gf_msg (this->name, GF_LOG_WARNING,
                                 GD_MSG_PMAP_REGISTRY_REMOVE_FAIL, 0, "Failed to"
@@ -342,7 +343,8 @@ pmap_registry_extend (xlator_t *this, int port, const char *brickname)
 
 int
 pmap_registry_remove (xlator_t *this, int port, const char *brickname,
-                      gf_pmap_port_type_t type, void *xprt)
+                      gf_pmap_port_type_t type, void *xprt,
+                      gf_boolean_t brick_disconnect)
 {
         struct pmap_registry *pmap = NULL;
         int                   p = 0;
@@ -389,11 +391,16 @@ remove:
          * can delete the entire entry.
          */
         if (!pmap->ports[p].xprt) {
-                brick_str = pmap->ports[p].brickname;
-                if (brick_str) {
-                        while (*brick_str != '\0') {
-                                if (*(brick_str++) != ' ') {
-                                        goto out;
+                /* If the signout call is being triggered by brick disconnect
+                 * then clean up all the bricks (in case of brick mux)
+                 */
+                if (!brick_disconnect) {
+                        brick_str = pmap->ports[p].brickname;
+                        if (brick_str) {
+                                while (*brick_str != '\0') {
+                                        if (*(brick_str++) != ' ') {
+                                                goto out;
+                                        }
                                 }
                         }
                 }
@@ -488,8 +495,8 @@ __gluster_pmap_signin (rpcsvc_request_t *req)
 {
         pmap_signin_req    args = {0,};
         pmap_signin_rsp    rsp  = {0,};
-        glusterd_brickinfo_t *brickinfo = NULL;
         int                ret = -1;
+        glusterd_brickinfo_t    *brickinfo = NULL;
 
         ret = xdr_to_generic (req->msg[0], &args,
                               (xdrproc_t)xdr_pmap_signin_req);
@@ -502,6 +509,10 @@ __gluster_pmap_signin (rpcsvc_request_t *req)
                                          GF_PMAP_PORT_BRICKSERVER, req->trans);
 
         ret = glusterd_get_brickinfo (THIS, args.brick, args.port, &brickinfo);
+        /* Update portmap status in brickinfo */
+        if (brickinfo)
+                brickinfo->port_registered = _gf_true;
+
 fail:
         glusterd_submit_reply (req, &rsp, NULL, 0, NULL,
                                (xdrproc_t)xdr_pmap_signin_rsp);
@@ -544,15 +555,20 @@ __gluster_pmap_signout (rpcsvc_request_t *req)
                 goto fail;
         }
         rsp.op_ret = pmap_registry_remove (THIS, args.port, args.brick,
-                                           GF_PMAP_PORT_BRICKSERVER, req->trans);
+                                           GF_PMAP_PORT_BRICKSERVER, req->trans,
+                                           _gf_false);
 
         ret = glusterd_get_brickinfo (THIS, args.brick, args.port, &brickinfo);
         if (args.rdma_port) {
                 snprintf(brick_path, PATH_MAX, "%s.rdma", args.brick);
                 rsp.op_ret = pmap_registry_remove (THIS, args.rdma_port,
                                 brick_path, GF_PMAP_PORT_BRICKSERVER,
-                                req->trans);
+                                req->trans, _gf_false);
         }
+        /* Update portmap status on brickinfo */
+        if (brickinfo)
+                brickinfo->port_registered = _gf_false;
+
         /* Clean up the pidfile for this brick given glusterfsd doesn't clean it
          * any more. This is required to ensure we don't end up with having
          * stale pid files in case a brick is killed from the backend
@@ -563,7 +579,28 @@ __gluster_pmap_signout (rpcsvc_request_t *req)
                         GLUSTERD_GET_BRICK_PIDFILE (pidfile, volinfo, brickinfo,
                                                     conf);
                         sys_unlink (pidfile);
-                        brickinfo->started_here = _gf_false;
+
+                        /* Setting the brick status to GF_BRICK_STOPPED to
+                         * ensure correct brick status is maintained on the
+                         * glusterd end when a brick is killed from the
+                         * backend */
+                        brickinfo->status = GF_BRICK_STOPPED;
+
+                        /* Remove brick from brick process if not already
+                         * removed in the brick op phase. This situation would
+                         * arise when the brick is killed explicitly from the
+                         * backend */
+                        ret = glusterd_brick_process_remove_brick (brickinfo);
+                        if (ret) {
+                                gf_msg_debug (this->name, 0, "Couldn't remove "
+                                              "brick %s:%s from brick process",
+                                              brickinfo->hostname,
+                                              brickinfo->path);
+                                /* Ignore 'ret' here since the brick might
+                                 * have already been deleted in brick op phase
+                                 */
+                                ret = 0;
+                        }
                 }
         }
 
