@@ -424,7 +424,8 @@ glusterd_rpcsvc_notify (rpcsvc_t *rpc, void *xl, rpcsvc_event_t event,
                 pthread_mutex_lock (&priv->xprt_lock);
                 list_del (&xprt->list);
                 pthread_mutex_unlock (&priv->xprt_lock);
-                pmap_registry_remove (this, 0, NULL, GF_PMAP_PORT_ANY, xprt);
+                pmap_registry_remove (this, 0, NULL, GF_PMAP_PORT_ANY, xprt,
+                                      _gf_false);
                 break;
         }
 
@@ -462,14 +463,12 @@ glusterd_rpcsvc_options_build (dict_t *options)
         int             ret = 0;
         uint32_t        backlog = 0;
 
-        ret = dict_get_uint32 (options, "transport.socket.listen-backlog",
-                               &backlog);
+        ret = dict_get_uint32 (options, "transport.listen-backlog", &backlog);
 
         if (ret) {
-                backlog = GLUSTERD_SOCKET_LISTEN_BACKLOG;
+                backlog = GLUSTERFS_SOCKET_LISTEN_BACKLOG;
                 ret = dict_set_uint32 (options,
-                                      "transport.socket.listen-backlog",
-                                      backlog);
+                                      "transport.listen-backlog", backlog);
                 if (ret)
                         goto out;
         }
@@ -1392,6 +1391,7 @@ init (xlator_t *this)
         struct stat        buf                        = {0,};
         char               storedir[PATH_MAX]         = {0,};
         char               workdir[PATH_MAX]          = {0,};
+        char               rundir[PATH_MAX]           = {0,};
         char               cmd_log_filename[PATH_MAX] = {0,};
         char              *mountbroker_root           = NULL;
         int                i                          = 0;
@@ -1403,6 +1403,7 @@ init (xlator_t *this)
         int32_t            workers                    = 0;
         gf_boolean_t       upgrade                    = _gf_false;
         gf_boolean_t       downgrade                  = _gf_false;
+        char              *localtime_logging          = NULL;
 
 #ifndef GF_DARWIN_HOST_OS
         {
@@ -1424,6 +1425,17 @@ init (xlator_t *this)
         }
 #endif
 
+        dir_data = dict_get (this->options, "run-directory");
+
+        if (!dir_data) {
+                /* Use default working dir */
+                strncpy (rundir, DEFAULT_VAR_RUN_DIRECTORY, PATH_MAX);
+        } else {
+                strncpy (rundir, dir_data->data, PATH_MAX);
+        }
+
+        dir_data = NULL;
+
         dir_data = dict_get (this->options, "working-directory");
 
         if (!dir_data) {
@@ -1468,6 +1480,11 @@ init (xlator_t *this)
                 GD_MSG_CURR_WORK_DIR_INFO, "Using %s as working directory",
                 workdir);
 
+        setenv ("DEFAULT_VAR_RUN_DIRECTORY", rundir, 1);
+        gf_msg (this->name, GF_LOG_INFO, 0,
+                GD_MSG_CURR_WORK_DIR_INFO, "Using %s as pid file working "
+                "directory", rundir);
+
         ret = glusterd_find_correct_var_run_dir (this, var_run_dir);
         if (ret) {
                 gf_msg (this->name, GF_LOG_CRITICAL, 0,
@@ -1478,6 +1495,7 @@ init (xlator_t *this)
 
         ret = glusterd_init_var_run_dirs (this, var_run_dir,
                                       GLUSTERD_DEFAULT_SNAPS_BRICK_DIR);
+
         if (ret) {
                 gf_msg (this->name, GF_LOG_CRITICAL, 0,
                         GD_MSG_CREATE_DIR_FAILED, "Unable to create "
@@ -1497,6 +1515,51 @@ init (xlator_t *this)
                 exit (1);
         }
 
+        ret = glusterd_init_var_run_dirs (this, rundir,
+                                          GLUSTERD_BITD_RUN_DIR);
+        if (ret) {
+                gf_msg (this->name, GF_LOG_CRITICAL, 0,
+                        GD_MSG_CREATE_DIR_FAILED, "Unable to create "
+                        "bitd running directory");
+                exit (1);
+        }
+
+        ret = glusterd_init_var_run_dirs (this, rundir,
+                                          GLUSTERD_SCRUB_RUN_DIR);
+        if (ret) {
+                gf_msg (this->name, GF_LOG_CRITICAL, 0,
+                        GD_MSG_CREATE_DIR_FAILED, "Unable to create "
+                        "scrub running directory");
+                exit (1);
+        }
+
+        ret = glusterd_init_var_run_dirs (this, rundir,
+                                          GLUSTERD_GLUSTERSHD_RUN_DIR);
+        if (ret) {
+                gf_msg (this->name, GF_LOG_CRITICAL, 0,
+                        GD_MSG_CREATE_DIR_FAILED, "Unable to create "
+                        "glustershd running directory");
+                exit (1);
+        }
+
+        ret = glusterd_init_var_run_dirs (this, rundir,
+                                          GLUSTERD_NFS_RUN_DIR);
+        if (ret) {
+                gf_msg (this->name, GF_LOG_CRITICAL, 0,
+                        GD_MSG_CREATE_DIR_FAILED, "Unable to create "
+                        "nfs running directory");
+                exit (1);
+        }
+
+        ret = glusterd_init_var_run_dirs (this, rundir,
+                                          GLUSTERD_QUOTAD_RUN_DIR);
+        if (ret) {
+                gf_msg (this->name, GF_LOG_CRITICAL, 0,
+                        GD_MSG_CREATE_DIR_FAILED, "Unable to create "
+                        "quota running directory");
+                exit (1);
+        }
+
         snprintf (cmd_log_filename, PATH_MAX, "%s/cmd_history.log",
                   DEFAULT_LOG_FILE_DIRECTORY);
         ret = gf_cmd_log_init (cmd_log_filename);
@@ -1520,6 +1583,19 @@ init (xlator_t *this)
                 exit (1);
         }
 
+        /*keeping individual volume pid file information in /var/run/gluster* */
+        snprintf (storedir, PATH_MAX, "%s/vols", rundir);
+
+        ret = sys_mkdir (storedir, 0777);
+
+        if ((-1 == ret) && (errno != EEXIST)) {
+                gf_msg (this->name, GF_LOG_CRITICAL, errno,
+                        GD_MSG_CREATE_DIR_FAILED,
+                        "Unable to create volume directory %s"
+                        " ,errno = %d", storedir, errno);
+                exit (1);
+        }
+
         snprintf (storedir, PATH_MAX, "%s/snaps", workdir);
 
         ret = sys_mkdir (storedir, 0777);
@@ -1725,12 +1801,14 @@ init (xlator_t *this)
         CDS_INIT_LIST_HEAD (&conf->volumes);
         CDS_INIT_LIST_HEAD (&conf->snapshots);
         CDS_INIT_LIST_HEAD (&conf->missed_snaps_list);
+        CDS_INIT_LIST_HEAD (&conf->brick_procs);
 
         pthread_mutex_init (&conf->mutex, NULL);
         conf->rpc = rpc;
         conf->uds_rpc = uds_rpc;
         conf->gfs_mgmt = &gd_brick_prog;
         strncpy (conf->workdir, workdir, PATH_MAX);
+        strncpy (conf->rundir, rundir, PATH_MAX);
 
         synclock_init (&conf->big_lock, SYNC_LOCK_RECURSIVE);
         pthread_mutex_init (&conf->xprt_lock, NULL);
@@ -1746,12 +1824,19 @@ init (xlator_t *this)
         if (ret)
                 goto out;
 
-         conf->base_port = GF_IANA_PRIV_PORTS_START;
-         if (dict_get_uint32(this->options, "base-port", &conf->base_port) == 0) {
+        conf->base_port = GF_IANA_PRIV_PORTS_START;
+        if (dict_get_uint32(this->options, "base-port", &conf->base_port) == 0) {
                 gf_msg (this->name, GF_LOG_INFO, 0,
                         GD_MSG_DICT_SET_FAILED,
                         "base-port override: %d", conf->base_port);
-         }
+        }
+        conf->mgmt_v3_lock_timeout = GF_LOCK_TIMER;
+        if (dict_get_uint32 (this->options, "lock-timer",
+                             &conf->mgmt_v3_lock_timeout) == 0) {
+                gf_msg (this->name, GF_LOG_INFO, 0,
+                        GD_MSG_DICT_SET_FAILED,
+                        "lock-timer override: %d", conf->mgmt_v3_lock_timeout);
+        }
 
         /* Set option to run bricks on valgrind if enabled in glusterd.vol */
         this->ctx->cmd_args.valgrind = valgrind;
@@ -1777,6 +1862,7 @@ init (xlator_t *this)
 
         this->private = conf;
         glusterd_mgmt_v3_lock_init ();
+        glusterd_mgmt_v3_lock_timer_init();
         glusterd_txn_opinfo_dict_init ();
 
         glusterd_shdsvc_build (&conf->shd_svc);
@@ -1847,6 +1933,25 @@ init (xlator_t *this)
         if (ret < 0)
                 goto out;
 
+        if (dict_get_str (conf->opts, GLUSTERD_LOCALTIME_LOGGING_KEY,
+                          &localtime_logging) == 0) {
+                int already_enabled = gf_log_get_localtime ();
+
+                if (strcmp (localtime_logging, "enable") == 0) {
+                        gf_log_set_localtime (1);
+                        if (!already_enabled)
+                                gf_msg (this->name, GF_LOG_INFO, 0,
+                                        GD_MSG_LOCALTIME_LOGGING_ENABLE,
+                                        "localtime logging enable");
+                } else if (strcmp (localtime_logging, "disable") == 0) {
+                        gf_log_set_localtime (0);
+                        if (already_enabled)
+                                gf_msg (this->name, GF_LOG_INFO, 0,
+                                        GD_MSG_LOCALTIME_LOGGING_DISABLE,
+                                        "localtime logging disable");
+                }
+        }
+
         conf->blockers = 0;
         /* If the peer count is less than 2 then this would be the best time to
          * spawn process/bricks that may need (re)starting since last time
@@ -1920,6 +2025,7 @@ fini (xlator_t *this)
                 gf_store_handle_destroy (conf->handle);
         glusterd_sm_tr_log_delete (&conf->op_sm_log);
         glusterd_mgmt_v3_lock_fini ();
+        glusterd_mgmt_v3_lock_timer_fini ();
         glusterd_txn_opinfo_dict_fini ();
         GF_FREE (conf);
 
@@ -1948,7 +2054,7 @@ notify (xlator_t *this, int32_t event, void *data, ...)
                 case GF_EVENT_POLLERR:
                         break;
 
-                case GF_EVENT_TRANSPORT_CLEANUP:
+                case GF_EVENT_CLEANUP:
                         break;
 
                 default:
@@ -2038,6 +2144,14 @@ struct volume_options options[] = {
           .type = GF_OPTION_TYPE_INT,
           .description = "Sets the base port for portmap query"
         },
+        { .key = {"mgmt-v3-lock-timeout"},
+          .type = GF_OPTION_TYPE_INT,
+          .max = 600,
+          .description = "Sets the mgmt-v3-lock-timeout for transactions."
+                         "Specifes the default timeout value after which "
+                         "lock acquired while performing transaction will "
+                         "be released."
+        },
         { .key = {"snap-brick-path"},
           .type = GF_OPTION_TYPE_STR,
           .description = "directory where the bricks for the snapshots will be created"
