@@ -40,7 +40,6 @@
 #include "events.h"
 
 #define GLUSTERD_TR_LOG_SIZE            50
-#define GLUSTERD_SOCKET_LISTEN_BACKLOG  128
 #define GLUSTERD_QUORUM_TYPE_KEY        "cluster.server-quorum-type"
 #define GLUSTERD_QUORUM_RATIO_KEY       "cluster.server-quorum-ratio"
 #define GLUSTERD_GLOBAL_OPT_VERSION     "global-option-version"
@@ -55,6 +54,8 @@
 #define GLUSTER_SHARED_STORAGE          "gluster_shared_storage"
 #define GLUSTERD_SHARED_STORAGE_KEY     "cluster.enable-shared-storage"
 #define GLUSTERD_BRICK_MULTIPLEX_KEY    "cluster.brick-multiplex"
+#define GLUSTERD_BRICKMUX_LIMIT_KEY     "cluster.max-bricks-per-process"
+#define GLUSTERD_LOCALTIME_LOGGING_KEY  "cluster.localtime-logging"
 
 #define GLUSTERD_SNAPS_MAX_HARD_LIMIT 256
 #define GLUSTERD_SNAPS_DEF_SOFT_LIMIT_PERCENT 90
@@ -129,6 +130,7 @@ typedef enum glusterd_op_ {
         GD_OP_DETACH_TIER_STATUS,
         GD_OP_DETACH_NOT_STARTED,
         GD_OP_REMOVE_TIER_BRICK,
+        GD_OP_ADD_TIER_BRICK,
         GD_OP_MAX,
 } glusterd_op_t;
 
@@ -146,6 +148,7 @@ typedef struct {
         gf_boolean_t             trace;
         uuid_t                   uuid;
         char                     workdir[PATH_MAX];
+        char                     rundir[PATH_MAX];
         rpcsvc_t                *rpc;
         glusterd_svc_t           shd_svc;
         glusterd_svc_t           nfs_svc;
@@ -155,6 +158,7 @@ typedef struct {
         struct pmap_registry    *pmap;
         struct cds_list_head     volumes;
         struct cds_list_head     snapshots; /*List of snap volumes */
+        struct cds_list_head     brick_procs; /* List of brick processes */
         pthread_mutex_t          xprt_lock;
         struct list_head         xprt_list;
         gf_store_handle_t       *handle;
@@ -170,6 +174,7 @@ typedef struct {
                                                  * cluster with no
                                                  * transaction ids */
 
+        dict_t                    *mgmt_v3_lock_timer;
         struct cds_list_head       mount_specs;
         pthread_t                  brick_thread;
         void                      *hooks_priv;
@@ -190,6 +195,7 @@ typedef struct {
         uint32_t                   generation;
         int32_t                    workers;
         uint32_t                   blockers;
+        uint32_t                   mgmt_v3_lock_timeout;
 } glusterd_conf_t;
 
 
@@ -197,6 +203,7 @@ typedef enum gf_brick_status {
         GF_BRICK_STOPPED,
         GF_BRICK_STARTED,
         GF_BRICK_STOPPING,
+        GF_BRICK_STARTING
 } gf_brick_status_t;
 
 struct glusterd_brickinfo {
@@ -229,11 +236,27 @@ struct glusterd_brickinfo {
          */
         uint16_t           group;
         uuid_t             jbr_uuid;
-        gf_boolean_t       started_here;
+
+        /* Below are used for handling the case of multiple bricks sharing
+           the backend filesystem */
+        uint64_t           statfs_fsid;
+        uint32_t           fs_share_count;
+        gf_boolean_t       port_registered;
+        gf_boolean_t       start_triggered;
+        pthread_mutex_t    restart_mutex;
 };
 
 typedef struct glusterd_brickinfo glusterd_brickinfo_t;
 
+struct glusterd_brick_proc {
+        int                     port;
+        uint32_t                brick_count;
+        struct cds_list_head    brick_proc_list;
+        struct cds_list_head    bricks;
+};
+
+typedef struct glusterd_brick_proc glusterd_brick_proc_t;
+
 struct gf_defrag_brickinfo_ {
         char *name;
         int   files;
@@ -560,6 +583,11 @@ typedef enum {
 #define GLUSTERD_VOL_SNAP_DIR_PREFIX "snaps"
 
 #define GLUSTERD_DEFAULT_SNAPS_BRICK_DIR     "/gluster/snaps"
+#define GLUSTERD_BITD_RUN_DIR                "/bitd"
+#define GLUSTERD_SCRUB_RUN_DIR               "/scrub"
+#define GLUSTERD_GLUSTERSHD_RUN_DIR          "/glustershd"
+#define GLUSTERD_NFS_RUN_DIR                 "/nfs"
+#define GLUSTERD_QUOTAD_RUN_DIR              "/quotad"
 #define GLUSTER_SHARED_STORAGE_BRICK_DIR     GLUSTERD_DEFAULT_WORKDIR"/ss_brick"
 #define GLUSTERD_VAR_RUN_DIR                 "/var/run"
 #define GLUSTERD_RUN_DIR                     "/run"
@@ -595,6 +623,19 @@ typedef ssize_t (*gd_serialize_t) (struct iovec outmsg, void *args);
                           volinfo->volname);                            \
         } while (0)
 
+#define GLUSTERD_GET_VOLUME_PID_DIR(path, volinfo, priv)                   \
+do {                                                                       \
+        if (volinfo->is_snap_volume) {                                     \
+                snprintf (path, PATH_MAX, "%s/snaps/%s/%s",                \
+                          priv->rundir,                                    \
+                          volinfo->snapshot->snapname, volinfo->volname);  \
+        } else {                                                           \
+                snprintf (path, PATH_MAX, "%s/vols/%s",                    \
+                          priv->rundir,                                    \
+                          volinfo->volname);                               \
+        }                                                                  \
+} while (0)
+
 #define GLUSTERD_GET_SNAP_DIR(path, snap, priv)                           \
                 snprintf (path, PATH_MAX, "%s/snaps/%s", priv->workdir,   \
                           snap->snapname);
@@ -647,20 +688,20 @@ typedef ssize_t (*gd_serialize_t) (struct iovec outmsg, void *args);
 #define GLUSTERD_GET_BRICK_PIDFILE(pidfile,volinfo,brickinfo, priv) do {      \
                 char exp_path[PATH_MAX] = {0,};                               \
                 char volpath[PATH_MAX]  = {0,};                               \
-                GLUSTERD_GET_VOLUME_DIR (volpath, volinfo, priv);             \
+                GLUSTERD_GET_VOLUME_PID_DIR (volpath, volinfo, priv);         \
                 GLUSTERD_REMOVE_SLASH_FROM_PATH (brickinfo->path, exp_path);  \
-                snprintf (pidfile, PATH_MAX, "%s/run/%s-%s.pid",              \
-                          volpath, brickinfo->hostname, exp_path);      \
+                snprintf (pidfile, PATH_MAX, "%s/%s-%s.pid",                  \
+                          volpath, brickinfo->hostname, exp_path);            \
         } while (0)
 
-#define GLUSTERD_GET_NFS_PIDFILE(pidfile,nfspath) {                     \
-                snprintf (pidfile, PATH_MAX, "%s/run/nfs.pid",          \
-                          nfspath);                                     \
+#define GLUSTERD_GET_NFS_PIDFILE(pidfile, nfspath, priv) {            \
+                snprintf (pidfile, PATH_MAX, "%s/nfs/nfs.pid",        \
+                          priv->rundir);                              \
         }
 
-#define GLUSTERD_GET_QUOTAD_PIDFILE(pidfile,quotadpath) {                     \
-                snprintf (pidfile, PATH_MAX, "%s/run/quotad.pid",          \
-                          quotadpath);                                     \
+#define GLUSTERD_GET_QUOTAD_PIDFILE(pidfile, quotadpath, priv) {         \
+                snprintf (pidfile, PATH_MAX, "%s/quotad/quotad.pid",     \
+                           priv->rundir);                                \
         }
 
 #define GLUSTERD_GET_QUOTA_CRAWL_PIDDIR(piddir, volinfo, type) do {           \
@@ -937,6 +978,9 @@ glusterd_handle_attach_tier (rpcsvc_request_t *req);
 int
 glusterd_handle_detach_tier (rpcsvc_request_t *req);
 
+int
+glusterd_handle_add_tier_brick (rpcsvc_request_t *req);
+
 int
 glusterd_handle_replace_brick (rpcsvc_request_t *req);
 
@@ -1126,6 +1170,7 @@ int glusterd_op_delete_volume (dict_t *dict);
 int manage_export_config (char *volname, char *value, char **op_errstr);
 
 int glusterd_op_add_brick (dict_t *dict, char **op_errstr);
+int glusterd_op_add_tier_brick (dict_t *dict, char **op_errstr);
 int glusterd_op_remove_brick (dict_t *dict, char **op_errstr);
 int glusterd_op_stage_add_brick (dict_t *dict, char **op_errstr,
                                  dict_t *rsp_dict);
