@@ -633,7 +633,7 @@ updatefn(dict_t *dict, char *key, data_t *value, void *data)
                  * not update their cache if the value of a xattr is a 0 byte
                  * data (i.e. "").
                  */
-                if (!strcmp (value->data, ""))
+                if (value->len == 1 && value->data[0] == '\0')
                         continue;
 
 		if (dict_set(u->dict, key, value) < 0) {
@@ -2587,21 +2587,21 @@ mdc_priv_dump (xlator_t *this)
         gf_proc_dump_add_section(key_prefix);
 
         gf_proc_dump_write("stat_hit_count", "%"PRId64,
-                           conf->mdc_counter.stat_hit);
+                           conf->mdc_counter.stat_hit.cnt);
         gf_proc_dump_write("stat_miss_count", "%"PRId64,
-                           conf->mdc_counter.stat_miss);
+                           conf->mdc_counter.stat_miss.cnt);
         gf_proc_dump_write("xattr_hit_count", "%"PRId64,
-                           conf->mdc_counter.xattr_hit);
+                           conf->mdc_counter.xattr_hit.cnt);
         gf_proc_dump_write("xattr_miss_count", "%"PRId64,
-                           conf->mdc_counter.xattr_miss);
+                           conf->mdc_counter.xattr_miss.cnt);
         gf_proc_dump_write("nameless_lookup_count", "%"PRId64,
-                           conf->mdc_counter.nameless_lookup);
+                           conf->mdc_counter.nameless_lookup.cnt);
         gf_proc_dump_write("negative_lookup_count", "%"PRId64,
-                           conf->mdc_counter.negative_lookup);
+                           conf->mdc_counter.negative_lookup.cnt);
         gf_proc_dump_write("stat_invalidations_received", "%"PRId64,
-                           conf->mdc_counter.stat_invals);
+                           conf->mdc_counter.stat_invals.cnt);
         gf_proc_dump_write("xattr_invalidations_received", "%"PRId64,
-                           conf->mdc_counter.xattr_invals);
+                           conf->mdc_counter.xattr_invals.cnt);
 
         return 0;
 }
@@ -2891,7 +2891,10 @@ reconfigure (xlator_t *this, dict_t *options)
 
 	GF_OPTION_RECONF ("cache-posix-acl", conf->cache_posix_acl, options, bool, out);
 	mdc_key_load_set (mdc_keys, "system.posix_acl_", conf->cache_posix_acl);
-	mdc_key_load_set (mdc_keys, "glusterfs.posix_acl.", conf->cache_posix_acl);
+	mdc_key_load_set (mdc_keys, GF_POSIX_ACL_ACCESS,
+			  conf->cache_posix_acl);
+	mdc_key_load_set (mdc_keys, GF_POSIX_ACL_DEFAULT,
+			  conf->cache_posix_acl);
 
         GF_OPTION_RECONF ("cache-swift-metadata", conf->cache_swift_metadata,
                           options, bool, out);
@@ -2962,7 +2965,10 @@ init (xlator_t *this)
 
 	GF_OPTION_INIT ("cache-posix-acl", conf->cache_posix_acl, bool, out);
 	mdc_key_load_set (mdc_keys, "system.posix_acl_", conf->cache_posix_acl);
-	mdc_key_load_set (mdc_keys, "glusterfs.posix_acl.", conf->cache_posix_acl);
+	mdc_key_load_set (mdc_keys, GF_POSIX_ACL_ACCESS,
+			  conf->cache_posix_acl);
+	mdc_key_load_set (mdc_keys, GF_POSIX_ACL_DEFAULT,
+			  conf->cache_posix_acl);
 
         GF_OPTION_INIT ("cache-swift-metadata",
                         conf->cache_swift_metadata, bool, out);
