@@ -1079,10 +1079,14 @@ client_setvolume_cbk (struct rpc_req *req, struct iovec *iov, int count, void *m
         int32_t               op_errno      = 0;
         gf_boolean_t          auth_fail     = _gf_false;
         uint32_t              lk_ver        = 0;
+        glusterfs_ctx_t      *ctx           = NULL;
 
         frame = myframe;
         this  = frame->this;
         conf  = this->private;
+        GF_VALIDATE_OR_GOTO (this->name, conf, out);
+        ctx = this->ctx;
+        GF_VALIDATE_OR_GOTO (this->name, ctx, out);
 
         if (-1 == req->rpc_status) {
                 gf_msg (frame->this->name, GF_LOG_WARNING, ENOTCONN,
@@ -1137,13 +1141,24 @@ client_setvolume_cbk (struct rpc_req *req, struct iovec *iov, int count, void *m
         if (op_ret < 0) {
                 gf_msg (this->name, GF_LOG_ERROR, op_errno,
                         PC_MSG_SETVOLUME_FAIL,
-                        "SETVOLUME on remote-host failed");
+                        "SETVOLUME on remote-host failed: %s", remote_error);
+
                 errno = op_errno;
                 if (remote_error &&
                     (strcmp ("Authentication failed", remote_error) == 0)) {
                         auth_fail = _gf_true;
                         op_ret = 0;
                 }
+                if ((op_errno == ENOENT) && this->ctx->cmd_args.subdir_mount &&
+                    (ctx->graph_id <= 1)) {
+                        /* A case of subdir not being present at the moment,
+                           ride on auth_fail framework to notify the error */
+                        /* Make sure this case is handled only in the new
+                           graph, so mount may fail in this case. In case
+                           of 'add-brick' etc, we need to continue retry */
+                        auth_fail = _gf_true;
+                        op_ret = 0;
+                }
                 if (op_errno == ESTALE) {
                         ret = client_notify_dispatch (this,
                                                       GF_EVENT_VOLFILE_MODIFIED,
@@ -1377,6 +1392,18 @@ client_setvolume (xlator_t *this, struct rpc_clnt *rpc)
                                 "'volfile-checksum'");
         }
 
+        if (this->ctx->cmd_args.subdir_mount) {
+                ret = dict_set_str (options, "subdir-mount",
+                                    this->ctx->cmd_args.subdir_mount);
+                if (ret) {
+                        gf_log (THIS->name, GF_LOG_ERROR,
+                                "Failed to set subdir_mount");
+                        /* It makes sense to fail, as per the CLI, we
+                           should be doing a subdir_mount */
+                        goto fail;
+                }
+        }
+
         ret = dict_set_int16 (options, "clnt-lk-version",
                               client_get_lk_ver (conf));
         if (ret < 0) {
