@@ -23,6 +23,7 @@
 #include "event.h"
 #include "events.h"
 #include "server-messages.h"
+#include "glusterfsd.h"
 
 rpcsvc_cbk_program_t server_cbk_prog = {
         .progname  = "Gluster Callback",
@@ -72,10 +73,8 @@ grace_time_handler (void *data)
         if (cancelled) {
 
                 /*
-                 * client must not be destroyed in gf_client_put(),
-                 * so take a ref.
+                 * ref has already been taken in server_rpc_notify()
                  */
-                gf_client_ref (client);
                 gf_client_put (client, &detached);
 
                 if (detached) /* reconnection did not happen :-( */
@@ -387,9 +386,6 @@ _check_for_auth_option (dict_t *d, char *k, data_t *v,
         int       ret           = 0;
         xlator_t *xl            = NULL;
         char     *tail          = NULL;
-        char     *tmp_addr_list = NULL;
-        char     *addr          = NULL;
-        char     *tmp_str       = NULL;
 
         xl = tmp;
 
@@ -418,37 +414,15 @@ _check_for_auth_option (dict_t *d, char *k, data_t *v,
                  * valid auth.allow.<xlator>
                  * Now we verify the ip address
                  */
-                if (!strcmp (v->data, "*")) {
-                        ret = 0;
-                        goto out;
-                }
-
-                tmp_addr_list = gf_strdup (v->data);
-                addr = strtok_r (tmp_addr_list, ",", &tmp_str);
-                if (!addr)
-                        addr = v->data;
-
-                while (addr) {
-                        if (valid_internet_address (addr, _gf_true)) {
-                                ret = 0;
-                        } else {
-                                ret = -1;
-                                gf_msg (xl->name, GF_LOG_ERROR, 0,
-                                        PS_MSG_INTERNET_ADDR_ERROR,
-                                        "internet address '%s'"
-                                        " does not conform to"
-                                        " standards.", addr);
-                                goto out;
-                        }
-                        if (tmp_str)
-                                addr = strtok_r (NULL, ",", &tmp_str);
-                        else
-                                addr = NULL;
-                }
+                ret = xlator_option_validate_addr_list (xl, "auth-*", v->data,
+                                                        NULL, NULL);
+                if (ret)
+                        gf_msg (xl->name, GF_LOG_ERROR, 0,
+                                PS_MSG_INTERNET_ADDR_ERROR,
+                                "internet address '%s' does not conform "
+                                "to standards.", v->data);
         }
 out:
-        GF_FREE (tmp_addr_list);
-
         return ret;
 }
 
@@ -541,13 +515,12 @@ server_rpc_notify (rpcsvc_t *rpc, void *xl, rpcsvc_event_t event,
                  * new transport will be created upon reconnect.
                  */
                 pthread_mutex_lock (&conf->mutex);
+                client = trans->xl_private;
                 list_del_init (&trans->list);
-                rpc_transport_unref (trans);
                 pthread_mutex_unlock (&conf->mutex);
 
-                client = trans->xl_private;
                 if (!client)
-                        break;
+                        goto unref_transport;
 
                 gf_msg (this->name, GF_LOG_INFO, 0,
                         PS_MSG_CLIENT_DISCONNECTING, "disconnecting connection"
@@ -580,18 +553,13 @@ server_rpc_notify (rpcsvc_t *rpc, void *xl, rpcsvc_event_t event,
                                           trans->myinfo.identifier,
                                           auth_path);
                         }
-                        gf_client_unref (client);
-                        break;
+
+                        /*
+                         * gf_client_unref will be done while handling
+                         * RPC_EVENT_TRANSPORT_DESTROY
+                         */
+                        goto unref_transport;
                 }
-                trans->xl_private = NULL;
-                server_connection_cleanup (this, client, INTERNAL_LOCKS);
-                gf_event (EVENT_CLIENT_DISCONNECT, "client_uid=%s;"
-                          "client_identifier=%s;server_identifier=%s;"
-                          "brick_path=%s",
-                          client->client_uid,
-                          trans->peerinfo.identifier,
-                          trans->myinfo.identifier,
-                          auth_path);
 
                 serv_ctx = server_ctx_get (client, this);
 
@@ -599,7 +567,7 @@ server_rpc_notify (rpcsvc_t *rpc, void *xl, rpcsvc_event_t event,
                         gf_msg (this->name, GF_LOG_INFO, 0,
                                 PS_MSG_SERVER_CTX_GET_FAILED,
                                 "server_ctx_get() failed");
-                        goto out;
+                        goto unref_transport;
                 }
 
                 grace_ts.tv_sec = conf->grace_timeout;
@@ -614,6 +582,12 @@ server_rpc_notify (rpcsvc_t *rpc, void *xl, rpcsvc_event_t event,
                                         "starting a grace timer for %s",
                                         client->client_uid);
 
+                                /* ref to protect against client destruction
+                                 * in RPCSVC_EVENT_TRANSPORT_DESTROY while
+                                 * we are starting a grace timer
+                                 */
+                                gf_client_ref (client);
+
                                 serv_ctx->grace_timer =
                                         gf_timer_call_after (this->ctx,
                                                              grace_ts,
@@ -622,13 +596,42 @@ server_rpc_notify (rpcsvc_t *rpc, void *xl, rpcsvc_event_t event,
                         }
                 }
                 UNLOCK (&serv_ctx->fdtable_lock);
+
+                ret = dict_get_str (this->options, "auth-path", &auth_path);
+                if (ret) {
+                        gf_msg (this->name, GF_LOG_WARNING, 0,
+                                PS_MSG_DICT_GET_FAILED,
+                                "failed to get auth-path");
+                        auth_path = NULL;
+                }
+
+                gf_event (EVENT_CLIENT_DISCONNECT, "client_uid=%s;"
+                          "client_identifier=%s;server_identifier=%s;"
+                          "brick_path=%s",
+                          client->client_uid,
+                          trans->peerinfo.identifier,
+                          trans->myinfo.identifier,
+                          auth_path);
+
+unref_transport:
+                /* rpc_transport_unref() causes a RPCSVC_EVENT_TRANSPORT_DESTROY
+                 * to be called in blocking manner
+                 * So no code should ideally be after this unref
+                 */
+                rpc_transport_unref (trans);
+
                 break;
+
         case RPCSVC_EVENT_TRANSPORT_DESTROY:
-                /*- conn obj has been disassociated from trans on first
-                 *  disconnect.
-                 *  conn cleanup and destruction is handed over to
-                 *  grace_time_handler or the subsequent handler that 'owns'
-                 *  the conn. Nothing left to be done here. */
+                client = trans->xl_private;
+                if (!client)
+                        break;
+
+                /* unref only for if (!client->lk_heal) */
+                if (!conf->lk_heal)
+                        gf_client_unref (client);
+
+                trans->xl_private = NULL;
                 break;
         default:
                 break;
@@ -1065,7 +1068,9 @@ init (xlator_t *this)
         if (ret)
                 conf->conf_dir = CONFDIR;
 
-        conf->child_up = _gf_false;
+        conf->child_status = GF_CALLOC (1, sizeof (struct _child_status),
+                                          gf_server_mt_child_status);
+        INIT_LIST_HEAD (&conf->child_status->status_list);
 
         /*ret = dict_get_str (this->options, "statedump-path", &statedump_path);
         if (!ret) {
@@ -1389,10 +1394,15 @@ server_process_child_event (xlator_t *this, int32_t event, void *data,
         pthread_mutex_lock (&conf->mutex);
         {
                 list_for_each_entry (xprt, &conf->xprt_list, list) {
-                        rpcsvc_callback_submit (conf->rpc, xprt,
-                                                &server_cbk_prog,
-                                                cbk_procnum,
-                                                NULL, 0, NULL);
+                        if (!xprt->xl_private) {
+                                continue;
+                        }
+                        if (xprt->xl_private->bound_xl == data) {
+                                rpcsvc_callback_submit (conf->rpc, xprt,
+                                                        &server_cbk_prog,
+                                                        cbk_procnum,
+                                                        NULL, 0, NULL);
+                        }
                 }
         }
         pthread_mutex_unlock (&conf->mutex);
@@ -1409,10 +1419,19 @@ notify (xlator_t *this, int32_t event, void *data, ...)
         server_conf_t    *conf        = NULL;
         rpc_transport_t  *xprt        = NULL;
         rpc_transport_t  *xp_next     = NULL;
+        xlator_t         *victim      = NULL;
+        xlator_t         *top         = NULL;
+        xlator_t         *travxl      = NULL;
+        xlator_list_t    **trav_p     = NULL;
+        struct  _child_status *tmp    = NULL;
+        gf_boolean_t     victim_found = _gf_false;
+        glusterfs_ctx_t  *ctx         = NULL;
 
         GF_VALIDATE_OR_GOTO (THIS->name, this, out);
         conf = this->private;
         GF_VALIDATE_OR_GOTO (this->name, conf, out);
+        victim = data;
+        ctx    = THIS->ctx;
 
         switch (event) {
         case GF_EVENT_UPCALL:
@@ -1441,7 +1460,24 @@ notify (xlator_t *this, int32_t event, void *data, ...)
 
         case GF_EVENT_CHILD_UP:
         {
-                conf->child_up = _gf_true;
+                list_for_each_entry (tmp, &conf->child_status->status_list,
+                                                                 status_list) {
+                        if (tmp->name == NULL)
+                                break;
+                        if (strcmp (tmp->name, victim->name) == 0)
+                                break;
+                }
+                if (tmp->name) {
+                        tmp->child_up = _gf_true;
+                } else {
+                        tmp  = GF_CALLOC (1, sizeof (struct _child_status),
+                                          gf_server_mt_child_status);
+                        INIT_LIST_HEAD (&tmp->status_list);
+                        tmp->name  = gf_strdup (victim->name);
+                        tmp->child_up = _gf_true;
+                        list_add_tail (&tmp->status_list,
+                                              &conf->child_status->status_list);
+                }
                 ret = server_process_child_event (this, event, data,
                                                   GF_CBK_CHILD_UP);
                 if (ret) {
@@ -1450,14 +1486,25 @@ notify (xlator_t *this, int32_t event, void *data, ...)
                                 "server_process_child_event failed");
                         goto out;
                 }
-
                 default_notify (this, event, data);
                 break;
         }
 
         case GF_EVENT_CHILD_DOWN:
         {
-                conf->child_up = _gf_false;
+                list_for_each_entry (tmp, &conf->child_status->status_list,
+                                                                  status_list) {
+                        if (strcmp (tmp->name, victim->name) == 0) {
+                                tmp->child_up = _gf_false;
+                                break;
+                        }
+                }
+                if (!tmp->name)
+                        gf_msg (this->name, GF_LOG_ERROR, 0,
+                                PS_MSG_CHILD_STATUS_FAILED,
+                                "No xlator %s is found in "
+                                "child status list", victim->name);
+
                 ret = server_process_child_event (this, event, data,
                                                   GF_CBK_CHILD_DOWN);
                 if (ret) {
@@ -1466,13 +1513,12 @@ notify (xlator_t *this, int32_t event, void *data, ...)
                                 "server_process_child_event failed");
                         goto out;
                 }
-
                 default_notify (this, event, data);
                 break;
 
         }
 
-        case GF_EVENT_TRANSPORT_CLEANUP:
+        case GF_EVENT_CLEANUP:
                 conf = this->private;
                 pthread_mutex_lock (&conf->mutex);
                 /*
@@ -1493,8 +1539,40 @@ notify (xlator_t *this, int32_t event, void *data, ...)
                                 rpc_transport_disconnect (xprt, _gf_false);
                         }
                 }
+                list_for_each_entry (tmp, &conf->child_status->status_list,
+                                                                 status_list) {
+                        if (strcmp (tmp->name, victim->name) == 0)
+                                break;
+                }
+                if (tmp->name && (strcmp (tmp->name, victim->name) == 0)) {
+                        GF_FREE (tmp->name);
+                        list_del (&tmp->status_list);
+                }
                 pthread_mutex_unlock (&conf->mutex);
-                /* NB: do *not* propagate anywhere else */
+                if (this->ctx->active) {
+                        top = this->ctx->active->first;
+                        LOCK (&ctx->volfile_lock);
+                                for (trav_p = &top->children; *trav_p;
+                                                   trav_p = &(*trav_p)->next) {
+                                        travxl = (*trav_p)->xlator;
+                                        if (travxl &&
+                                                   strcmp (travxl->name, victim->name) == 0) {
+                                                victim_found = _gf_true;
+                                                break;
+                                        }
+                                }
+                                if (victim_found)
+                                        glusterfs_delete_volfile_checksum (ctx,
+                                                 victim->volfile_id);
+                        UNLOCK (&ctx->volfile_lock);
+                        if (victim_found)
+                                (*trav_p) = (*trav_p)->next;
+                        glusterfs_mgmt_pmap_signout (ctx,
+                                                     victim->name);
+                        glusterfs_autoscale_threads (THIS->ctx, -1);
+                        default_notify (victim, GF_EVENT_CLEANUP, data);
+
+                }
                 break;
 
         default:
@@ -1533,6 +1611,10 @@ struct volume_options options[] = {
                     "socket*([ \t]),*([ \t])rdma"},
           .type  = GF_OPTION_TYPE_STR
         },
+        { .key   = {"transport.listen-backlog"},
+          .type  = GF_OPTION_TYPE_INT,
+          .default_value = "10",
+        },
         { .key   = {"volume-filename.*"},
           .type  = GF_OPTION_TYPE_PATH,
         },
