@@ -50,6 +50,7 @@
 #include "glusterfs3-xdr.h"
 #include "hashfn.h"
 #include "glusterfs-acl.h"
+#include "posix-gfid-path.h"
 #include "events.h"
 #include "glusterfsd.h"
 #include <sys/types.h>
@@ -110,6 +111,7 @@ posix_handle_georep_xattrs (call_frame_t *frame, const char *name,
         static const char *georep_xattr[]   = { "*.glusterfs.*.stime",
                                                 "*.glusterfs.*.xtime",
                                                 "*.glusterfs.*.entry_stime",
+                                                "*.glusterfs.volume-mark.*",
                                                 NULL
                                               };
         if (frame && frame->root) {
@@ -148,26 +150,10 @@ out:
         return ret;
 }
 
-
-static gf_boolean_t
-_is_in_array (char **str_array, char *str)
-{
-        int i = 0;
-
-        if (!str)
-                return _gf_false;
-
-        for (i = 0; str_array[i]; i++) {
-                if (strcmp (str, str_array[i]) == 0)
-                        return _gf_true;
-        }
-        return _gf_false;
-}
-
 static gf_boolean_t
 posix_xattr_ignorable (char *key)
 {
-        return _is_in_array (posix_ignore_xattrs, key);
+        return gf_get_index_by_elem (posix_ignore_xattrs, key) >= 0;
 }
 
 static int
@@ -515,6 +501,11 @@ _posix_xattr_get_set (dict_t *xattr_req, char *key, data_t *data,
         } else if (strcmp(key, GF_REQUEST_LINK_COUNT_XDATA) == 0) {
                 ret = dict_set (filler->xattr,
                                 GF_REQUEST_LINK_COUNT_XDATA, data);
+        } else if (strcmp (key, GF_GET_SIZE) == 0) {
+                if (filler->stbuf && IA_ISREG (filler->stbuf->ia_type)) {
+                        ret = dict_set_uint64 (filler->xattr, GF_GET_SIZE,
+                                               filler->stbuf->ia_size);
+                }
         } else {
                 ret = _posix_xattr_get_set_from_backend (filler, key);
         }
@@ -680,6 +671,7 @@ posix_pstat (xlator_t *this, uuid_t gfid, const char *path,
         struct stat  lstatbuf = {0, };
         struct iatt  stbuf = {0, };
         int          ret = 0;
+        int          op_errno = 0;
         struct posix_private *priv = NULL;
 
 
@@ -691,22 +683,12 @@ posix_pstat (xlator_t *this, uuid_t gfid, const char *path,
                 posix_fill_gfid_path (this, path, &stbuf);
 
         ret = sys_lstat (path, &lstatbuf);
-
-        if (ret != 0) {
-                if (ret == -1) {
-                        if (errno != ENOENT)
-                                gf_msg (this->name, GF_LOG_WARNING, errno,
-                                        P_MSG_LSTAT_FAILED,
-                                        "lstat failed on %s",
-                                        path);
-                } else {
-                        // may be some backend filesytem issue
-                        gf_msg (this->name, GF_LOG_ERROR, 0, P_MSG_LSTAT_FAILED,
-                                "lstat failed on %s and return value is %d "
-                                "instead of -1. Please see dmesg output to "
-                                "check whether the failure is due to backend "
-                                "filesystem issue", path, ret);
-                        ret = -1;
+        if (ret == -1) {
+                if (errno != ENOENT) {
+                        op_errno = errno;
+                        gf_msg (this->name, GF_LOG_WARNING, errno,
+                                P_MSG_LSTAT_FAILED, "lstat failed on %s", path);
+                        errno = op_errno;/*gf_msg could have changed errno*/
                 }
                 goto out;
         }
@@ -767,7 +749,7 @@ _handle_list_xattr (dict_t *xattr_req, const char *real_path, int fdnum,
         while (remaining_size > 0) {
                 key = list + list_offset;
 
-                if (_is_in_array (list_xattr_ignore_xattrs, key))
+                if (gf_get_index_by_elem (list_xattr_ignore_xattrs, key) >= 0)
                         goto next;
 
                 if (posix_special_xattr (marker_xattrs, key))
@@ -776,6 +758,9 @@ _handle_list_xattr (dict_t *xattr_req, const char *real_path, int fdnum,
                 if (posix_handle_georep_xattrs (NULL, key, NULL, _gf_false))
                         goto next;
 
+                if (posix_is_gfid2path_xattr (key))
+                        goto next;
+
                 if (dict_get (filler->xattr, key))
                         goto next;
 
@@ -871,6 +856,12 @@ posix_gfid_set (xlator_t *this, const char *path, loc_t *loc, dict_t *xattr_req)
                         loc->path);
                 goto out;
         }
+        if (gf_uuid_is_null (uuid_req)) {
+                gf_msg (this->name, GF_LOG_ERROR, EINVAL, P_MSG_NULL_GFID,
+                        "gfid is null for %s", loc ? loc->path : "");
+                ret = -1;
+                goto out;
+        }
 
         ret = sys_lsetxattr (path, GFID_XATTR_KEY, uuid_req, 16, XATTR_CREATE);
         if (ret == -1) {
@@ -1150,10 +1141,18 @@ posix_handle_pair (xlator_t *this, const char *real_path,
 {
         int sys_ret = -1;
         int ret     = 0;
+#ifdef GF_DARWIN_HOST_OS
+        const int error_code = EINVAL;
+#else
+        const int error_code = EEXIST;
+#endif
 
         if (XATTR_IS_PATHINFO (key)) {
                 ret = -EACCES;
                 goto out;
+        } else if (posix_is_gfid2path_xattr (key)) {
+                ret = -ENOTSUP;
+                goto out;
         } else if (ZR_FILE_CONTENT_REQUEST(key)) {
                 ret = posix_set_file_contents (this, real_path, key, value,
                                                flags);
@@ -1181,37 +1180,20 @@ posix_handle_pair (xlator_t *this, const char *real_path,
                                                 real_path);
                                 }
                         } else {
-
-#ifdef GF_DARWIN_HOST_OS
-                                if (errno == EINVAL) {
-                                        gf_msg_debug (this->name, 0, "%s: key:"
-                                                      "%s flags: %u length:%d "
-                                                      "error:%s", real_path,
-                                                      key, flags, value->len,
-                                                      strerror (errno));
-                                } else {
-                                        gf_msg (this->name, GF_LOG_ERROR,
-                                                errno, P_MSG_XATTR_FAILED,
-                                                "%s: key:%s flags: "
-                                                "%u length:%d",
-                                                real_path, key, flags,
-                                                value->len);
-
-#else /* ! DARWIN */
-                                if (errno == EEXIST)
+                                if (errno == error_code) {
                                         gf_msg_debug (this->name, 0,
                                                       "%s: key:%s"
                                                       "flags: %u length:%d",
                                                       real_path, key, flags,
                                                       value->len);
-                                else
+                                } else {
                                         gf_msg (this->name, GF_LOG_ERROR, errno,
                                                 P_MSG_XATTR_FAILED, "%s: key:%s"
                                                 "flags: %u length:%d",
                                                 real_path, key, flags,
                                                 value->len);
-#endif /* DARWIN */
                                 }
+                        }
 
                         goto out;
                 }
@@ -1230,6 +1212,9 @@ posix_fhandle_pair (xlator_t *this, int fd,
         if (XATTR_IS_PATHINFO (key)) {
                 ret = -EACCES;
                 goto out;
+        } else if (posix_is_gfid2path_xattr (key)) {
+                ret = -ENOTSUP;
+                goto out;
         } else if (!strncmp(key, POSIX_ACL_ACCESS_XATTR, strlen(key))
                    && stbuf && IS_DHT_LINKFILE_MODE (stbuf)) {
                 goto out;
@@ -1423,6 +1408,7 @@ posix_janitor_thread_proc (void *data)
         THIS = this;
 
         while (1) {
+
                 time (&now);
                 if ((now - priv->last_landfill_check) > priv->janitor_sleep_duration) {
                         gf_msg_trace (this->name, 0,
@@ -1469,7 +1455,8 @@ posix_spawn_janitor_thread (xlator_t *this)
         {
                 if (!priv->janitor_present) {
                         ret = gf_thread_create (&priv->janitor, NULL,
-                                                posix_janitor_thread_proc, this);
+                                                posix_janitor_thread_proc,
+                                                this, "posixjan");
 
                         if (ret < 0) {
                                 gf_msg (this->name, GF_LOG_ERROR, errno,
@@ -1533,23 +1520,21 @@ posix_gfid_heal (xlator_t *this, const char *path, loc_t *loc, dict_t *xattr_req
         struct stat  stat = {0, };
 
         if (!xattr_req)
-                goto out;
+                return 0;
 
-        if (sys_lstat (path, &stat) != 0)
-                goto out;
+        if (sys_lstat (path, &stat) != 0) {
+                return -errno;
+        }
 
         ret = sys_lgetxattr (path, GFID_XATTR_KEY, uuid_curr, 16);
         if (ret != 16) {
                 if (is_fresh_file (&stat)) {
-                        ret = -1;
-                        errno = ENOENT;
-                        goto out;
+                        return -ENOENT;
                 }
         }
 
-        ret = posix_gfid_set (this, path, loc, xattr_req);
-out:
-        return ret;
+        posix_gfid_set (this, path, loc, xattr_req);
+        return 0;
 }
 
 
@@ -1852,9 +1837,11 @@ posix_health_check_thread_proc (void *data)
         xlator_list_t       **trav_p             = NULL;
         int                   count              = 0;
         gf_boolean_t          victim_found       = _gf_false;
+        glusterfs_ctx_t      *ctx                = NULL;
 
         this = data;
         priv = this->private;
+        ctx  = THIS->ctx;
 
         /* prevent races when the interval is updated */
         interval = priv->health_check_interval;
@@ -1869,15 +1856,15 @@ posix_health_check_thread_proc (void *data)
                 ret = sleep (interval);
                 if (ret > 0)
                         break;
-
                 /* prevent thread errors while doing the health-check(s) */
                 pthread_setcancelstate (PTHREAD_CANCEL_DISABLE, NULL);
 
                 /* Do the health-check.*/
                 ret = posix_fs_health_check (this);
-                if (ret < 0)
+                if (ret < 0 && priv->health_check_active)
                         goto abort;
-
+                if (!priv->health_check_active)
+                        goto out;
                 pthread_setcancelstate (PTHREAD_CANCEL_ENABLE, NULL);
         }
 
@@ -1904,10 +1891,12 @@ abort:
         */
         if (this->ctx->active) {
                 top = this->ctx->active->first;
-                for (trav_p = &top->children; *trav_p;
+                LOCK (&ctx->volfile_lock);
+                        for (trav_p = &top->children; *trav_p;
                                                trav_p = &(*trav_p)->next) {
-                        count++;
-                }
+                                count++;
+                        }
+                UNLOCK (&ctx->volfile_lock);
         }
 
         if (count == 1) {
@@ -1927,20 +1916,21 @@ abort:
                         kill (getpid(), SIGKILL);
 
         } else {
-                for (trav_p = &top->children; *trav_p;
-                     trav_p = &(*trav_p)->next) {
-                        victim = (*trav_p)->xlator;
-                        if (victim &&
-                            strcmp (victim->name, priv->base_path) == 0) {
-                                victim_found = _gf_true;
-                                break;
+                LOCK (&ctx->volfile_lock);
+                        for (trav_p = &top->children; *trav_p;
+                             trav_p = &(*trav_p)->next) {
+                                victim = (*trav_p)->xlator;
+                                if (victim &&
+                                         strcmp (victim->name, priv->base_path) == 0) {
+                                        victim_found = _gf_true;
+                                        break;
+                                }
                         }
-                }
+                UNLOCK (&ctx->volfile_lock);
                 if (victim_found) {
-                        top->notify (top, GF_EVENT_TRANSPORT_CLEANUP, victim);
-                        glusterfs_mgmt_pmap_signout (glusterfsd_ctx,
-                                                     priv->base_path);
-                        glusterfs_autoscale_threads (THIS->ctx, -1);
+                        gf_log (THIS->name, GF_LOG_INFO, "detaching not-only "
+                                " child %s", priv->base_path);
+                        top->notify (top, GF_EVENT_CLEANUP, victim);
                 }
         }
 
@@ -1968,7 +1958,8 @@ posix_spawn_health_check_thread (xlator_t *xl)
                         goto unlock;
 
                 ret = gf_thread_create (&priv->health_check, NULL,
-                                        posix_health_check_thread_proc, xl);
+                                        posix_health_check_thread_proc,
+                                        xl, "posixhc");
                 if (ret < 0) {
                         priv->health_check_interval = 0;
                         priv->health_check_active = _gf_false;
@@ -2136,20 +2127,31 @@ posix_fetch_signature_xattr (char *real_path,
         int32_t ret = 0;
         char    *memptr    = NULL;
         ssize_t  xattrsize = 0;
+        char     val_buf[2048] = {0,};
+        gf_boolean_t have_val  = _gf_false;
 
-        xattrsize = sys_lgetxattr (real_path, key, NULL, 0);
-        if ((xattrsize == -1) && ((errno == ENOATTR) || (errno == ENODATA)))
-                return 0;
-        if (xattrsize == -1)
-                goto error_return;
-
+        xattrsize = sys_lgetxattr (real_path, key, val_buf,
+                                                           sizeof(val_buf) - 1);
+        if (xattrsize >= 0) {
+                have_val = _gf_true;
+        } else {
+                if (errno == ERANGE)
+                        xattrsize = sys_lgetxattr (real_path, key, NULL, 0);
+                if ((errno == ENOATTR) || (errno == ENODATA))
+                        return 0;
+                if (xattrsize == -1)
+                        goto error_return;
+        }
         memptr = GF_CALLOC (xattrsize + 1, sizeof (char), gf_posix_mt_char);
         if (!memptr)
                 goto error_return;
-        ret = sys_lgetxattr (real_path, key, memptr, xattrsize);
-        if (ret == -1)
-                goto freemem;
-
+        if (have_val) {
+                memcpy (memptr, val_buf, xattrsize);
+        } else {
+                ret = sys_lgetxattr (real_path, key, memptr, xattrsize);
+                if (ret == -1)
+                        goto freemem;
+        }
         ret = dict_set_dynptr (xattr, (char *)key, memptr, xattrsize);
         if (ret)
                 goto freemem;
@@ -2269,6 +2271,102 @@ posix_fdget_objectsignature (int fd, dict_t *xattr)
         return -EINVAL;
 }
 
+/*
+ * posix_resolve_dirgfid_to_path:
+ *       It converts given dirgfid to path by doing recursive readlinks at the
+ *  backend. If bname is given, it suffixes bname to dir path to form the
+ *  complete path else it doesn't. It allocates memory for the path and is
+ *  caller's responsibility to free the same. If bname is NULL and pargfid
+ *  is ROOT, then it returns "/"
+ **/
+
+int32_t
+posix_resolve_dirgfid_to_path (const uuid_t dirgfid, const char *brick_path,
+                               const char *bname, char **path)
+{
+        char             *linkname                  = NULL;
+        char             *dir_handle                = NULL;
+        char             *pgfidstr                  = NULL;
+        char             *saveptr                   = NULL;
+        ssize_t           len                       = 0;
+        int               ret                       = 0;
+        uuid_t            tmp_gfid                  = {0, };
+        uuid_t            pargfid                   = {0, };
+        char              gpath[PATH_MAX]           = {0,};
+        char              result[PATH_MAX]          = {0,};
+        char              result1[PATH_MAX]         = {0,};
+        char             *dir_name                  = NULL;
+        char              pre_dir_name[PATH_MAX]    = {0,};
+        xlator_t         *this                      = NULL;
+
+        this = THIS;
+        GF_ASSERT (this);
+
+        gf_uuid_copy (pargfid, dirgfid);
+        if (!path || gf_uuid_is_null (pargfid)) {
+                ret = -1;
+                goto out;
+        }
+
+        if (__is_root_gfid (pargfid)) {
+                if (bname) {
+                        snprintf (result, PATH_MAX, "/%s", bname);
+                        *path = gf_strdup (result);
+                } else {
+                        *path = gf_strdup ("/");
+                }
+                return ret;
+        }
+
+        dir_handle = alloca (PATH_MAX);
+        linkname   = alloca (PATH_MAX);
+        (void) snprintf (gpath, PATH_MAX, "%s/.glusterfs/", brick_path);
+
+        while (!(__is_root_gfid (pargfid))) {
+                snprintf (dir_handle, PATH_MAX, "%s/%02x/%02x/%s", gpath,
+                          pargfid[0], pargfid[1], uuid_utoa (pargfid));
+
+                len = sys_readlink (dir_handle, linkname, PATH_MAX);
+                if (len < 0) {
+                        gf_msg (this->name, GF_LOG_ERROR, errno,
+                                P_MSG_READLINK_FAILED,
+                                "could not read the "
+                                "link from the gfid handle %s", dir_handle);
+                        ret = -1;
+                        goto out;
+                }
+
+                linkname[len] = '\0';
+
+                pgfidstr = strtok_r (linkname + strlen("../../00/00/"), "/",
+                                     &saveptr);
+                dir_name = strtok_r (NULL, "/", &saveptr);
+
+                if (strlen(pre_dir_name) != 0) { /* Remove '/' at the end */
+                        snprintf (result, PATH_MAX, "%s/%s", dir_name,
+                                  pre_dir_name);
+                } else {
+                        snprintf (result, PATH_MAX, "%s", dir_name);
+                }
+
+                strncpy (pre_dir_name, result, sizeof(pre_dir_name));
+
+                gf_uuid_parse (pgfidstr, tmp_gfid);
+                gf_uuid_copy (pargfid, tmp_gfid);
+        }
+
+        if (bname) {
+                snprintf (result1, PATH_MAX, "/%s/%s", result, bname);
+        } else {
+                snprintf (result1, PATH_MAX, "/%s", result);
+        }
+
+        *path = gf_strdup (result1);
+
+out:
+        return ret;
+}
+
 posix_inode_ctx_t *
 __posix_inode_ctx_get (inode_t *inode, xlator_t *this)
 {
@@ -2358,3 +2456,11 @@ posix_inode_ctx_get_all (inode_t *inode, xlator_t *this,
 
         return ret;
 }
+
+gf_boolean_t
+posix_is_bulk_removexattr (char *name, dict_t *xdata)
+{
+        if (name && (strlen (name) == 0) && xdata)
+                return _gf_true;
+        return _gf_false;
+}
